==> idkit-0.3.1.tar.gz: lib/src/function/image/func_image_watermark.dart <==
import 'dart:math';
import 'dart:ui';
import 'package:flutter/rendering.dart';
import 'package:idkit/idkit.dart';

/// Image Watermark Extension.
extension WMarkImage on Image {
  /// Image Watermark Extension
  Future<Image> addWaterMarkImage(
    Image wmImg, {
    WMLocation location = WMLocation.tile,
    double opacity = 0.3,
    double sizeScale = 1,
    double space = 0,
    double vOffset = 20,
    double hOffset = 20,
    double angle = 0,
    bool srcDispose = false,
    bool wmDispose = false,
  }) async {
    final PictureRecorder recorder = PictureRecorder();
    final Canvas canvas = Canvas(recorder, rect);
    final Picture picture = recorder.endRecording();
    canvas.drawImage(this, Offset.zero, Paint());
    _watermakrLayer(
        canvas, wmImg, location, opacity, sizeScale, vOffset, hOffset, angle);
    if (srcDispose) dispose();
    if (wmDispose) wmImg.dispose();
    return picture.toImageSync(width, height);
  }

  /// Image Watermark Extension
  Future<Image> addWaterMarkText(
    String text, {
    WMLocation location = WMLocation.tile,
    TextStyle style = const TextStyle(),
    int? maxLines,
    double? minWidth,
    double? maxWidth,
    double opacity = 0.3,
    double sizeScale = 1,
    double space = 0,
    double vOffset = 20,
    double hOffset = 20,
    double angle = 0,
    bool isDispose = false,
  }) async {
    final PictureRecorder recorder = PictureRecorder();
    final Canvas canvas = Canvas(recorder, rect);
    final Picture picture = recorder.endRecording();
    canvas.drawImage(this, Offset.zero, Paint());
    final Image tImage = _textPaint(
      text,
      style,
      minWidth: minWidth ?? 0,
      maxWidth: maxWidth ?? double.infinity,
      maxLines: maxLines ?? 6,
    );
    _watermakrLayer(
        canvas, tImage, location, opacity, sizeScale, vOffset, hOffset, angle);
    if (isDispose) dispose();
    tImage.dispose();
    return picture.toImageSync(width, height);
  }

  /// Image Watermark Extension
  Image addWaterMarkImageSync(
    Image wmImg, {
    WMLocation location = WMLocation.tile,
    double opacity = 0.3,
    double sizeScale = 1,
    double hOffset = 20,
    double vOffset = 20,
    double angle = 0,
    bool srcDispose = false,
    bool wmDispose = false,
  }) {
    final PictureRecorder recorder = PictureRecorder();
    final Canvas canvas = Canvas(recorder, rect);
    canvas.drawImage(this, Offset.zero, Paint());
    _watermakrLayer(
        canvas, wmImg, location, opacity, sizeScale, hOffset, vOffset, angle);
    final Picture picture = recorder.endRecording();
    if (srcDispose) dispose();
    if (wmDispose) wmImg.dispose();
    return picture.toImageSync(width, height);
  }

  /// Image Watermark Extension
  Image addWaterMarkTextSync(
    String text, {
    WMLocation location = WMLocation.tile,
    TextStyle style = const TextStyle(),
    int? maxLines,
    double? minWidth,
    double? maxWidth,
    double opacity = 0.3,
    double sizeScale = 1,
    double vOffset = 20,
    double hOffset = 20,
    double angle = 0,
    bool isDispose = false,
  }) {
    final PictureRecorder recorder = PictureRecorder();
    final Canvas canvas = Canvas(recorder, rect);
    canvas.drawImage(this, Offset.zero, Paint());
    final Image tImage = _textPaint(
      text,
      style,
      minWidth: minWidth ?? 0,
      maxWidth: maxWidth ?? double.infinity,
      maxLines: maxLines ?? 6,
    );
    _watermakrLayer(
        canvas, tImage, location, opacity, sizeScale, vOffset, hOffset, angle);
    final Picture picture = recorder.endRecording();
    if (isDispose) dispose();
    tImage.dispose();
    return picture.toImageSync(width, height);
  }

  /// Text drawing
  Image _textPaint(
    String text,
    TextStyle style, {
    double minWidth = 0,
    double maxWidth = double.infinity,
    int maxLines = 6,
  }) {
    final TextPainter painter = TextPainter();
    painter.text = TextSpan(text: text, style: style);
    painter.textDirection = TextDirection.ltr;
    painter.maxLines = maxLines;
    painter.layout(minWidth: minWidth, maxWidth: maxWidth);
    final Rect tRect = Offset.zero & Size(painter.width, painter.height);
    final PictureRecorder recorder = PictureRecorder();
    final Canvas canvas = Canvas(recorder, tRect);
    canvas.drawRect(tRect, Paint()..color = '#00000000'.aHexColor);
    painter.paint(canvas, Offset.zero);
    final Picture picture = recorder.endRecording();
    return picture.toImageSync(tRect.width.toInt(), tRect.height.toInt());
  }

  /// Watermark layer drawing.
  void _watermakrLayer(
    Canvas canvas,
    Image wmImg,
    WMLocation location,
    double opacity,
    double sizeScale,
    double hOffset,
    double vOffset,
    double angle,
  ) {
    final ColorFilter colorFilter = ColorFilter.matrix(
        [1, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, opacity, 0]);
    final Image hwmImg = wmImg.scaleSync(scale: sizeScale);
    Paint paint = Paint();
    paint.colorFilter = colorFilter;
    switch (location) {
      case WMLocation.topLeft:
        _canvasRotation(canvas, Offset.zero, angle);
        canvas.drawImage(hwmImg, Offset(hOffset, vOffset), paint);
        break;
      case WMLocation.topCenter:
        final double translateDx = width * 0.5 + hOffset;
        _canvasRotation(canvas, Offset(translateDx, vOffset), angle);
        final int wMwidth = hwmImg.width;
        final double dx = (width - wMwidth) * 0.5 + hOffset;
        canvas.drawImage(hwmImg, Offset(dx, vOffset), paint);
        break;
      case WMLocation.topRight:
        final double translateDx = width - hOffset;
        _canvasRotation(canvas, Offset(translateDx, vOffset), angle);
        final int wMwidth = hwmImg.width;
        final double dx = width - (wMwidth + hOffset);
        canvas.drawImage(hwmImg, Offset(dx, vOffset), paint);
        break;
      case WMLocation.centerLeft:
        final double translateDy = height * 0.5 + vOffset;
        _canvasRotation(canvas, Offset(hOffset, translateDy), angle);
        final int wMHeight = hwmImg.height;
        final double dy = (height - wMHeight) * 0.5 + vOffset;
        canvas.drawImage(hwmImg, Offset(hOffset, dy), paint);
        break;
      case WMLocation.center:
        final double translateDx = width * 0.5 + hOffset;
        final double translateDy = height * 0.5 + vOffset;
        _canvasRotation(canvas, Offset(translateDx, translateDy), angle);
        final int wMwidth = hwmImg.width;
        final double dx = (width - wMwidth) * 0.5 + hOffset;
        final int wMHeight = hwmImg.height;
        final double dy = (height - wMHeight) * 0.5 + vOffset;
        canvas.drawImage(hwmImg, Offset(dx, dy), paint);
        break;
      case WMLocation.centerRight:
        final double translateDx = width - hOffset;
        final double translateDy = height * 0.5 + vOffset;
        _canvasRotation(canvas, Offset(translateDx, translateDy), angle);
        final int wMwidth = hwmImg.width;
        final double dx = width - (wMwidth + hOffset);
        final int wMHeight = hwmImg.height;
        final double dy = (height - wMHeight) * 0.5 + vOffset;
        canvas.drawImage(hwmImg, Offset(dx, dy), paint);
        break;
      case WMLocation.bottomLeft:
        final double translateDy = height - vOffset;
        _canvasRotation(canvas, Offset(hOffset, translateDy), angle);
        final int wMHeight = hwmImg.height;
        final double dy = height - (wMHeight + vOffset);
        canvas.drawImage(hwmImg, Offset(hOffset, dy), paint);
        break;
      case WMLocation.bottomCenter:
        final double translateDx = width * 0.5 + hOffset;
        final double translateDy = height - vOffset;
        _canvasRotation(canvas, Offset(translateDx, translateDy), angle);
        final int wMwidth = hwmImg.width;
        final double dx = (width - wMwidth) * 0.5 + hOffset;
        final int wMHeight = hwmImg.height;
        final double dy = height - (wMHeight + vOffset);
        canvas.drawImage(hwmImg, Offset(dx, dy), paint);
        break;
      case WMLocation.bottomRight:
        final double translateDx = width - hOffset;
        final double translateDy = height - vOffset;
        _canvasRotation(canvas, Offset(translateDx, translateDy), angle);
        final int wMwidth = hwmImg.width;
        final double dx = width - (wMwidth + hOffset);
        final int wMHeight = hwmImg.height;
        final double dy = height - (wMHeight + vOffset);
        canvas.drawImage(hwmImg, Offset(dx, dy), paint);
        break;
      default:
        final Matrix4 matrix4 = Matrix4.identity();
        matrix4.rotateZ((angle / 180) * pi);
        matrix4.translate(hOffset, vOffset);
        matrix4.scale(sizeScale, sizeScale);
        paint.shader = ImageShader(
            wmImg, TileMode.repeated, TileMode.repeated, matrix4.storage);
        canvas.drawRect(rect, paint);
    }
    // destroy
    hwmImg.dispose();
  }

  /// Canvas Rotation Handling.
  void _canvasRotation(Canvas canvas, Offset roationCenter, double angle) {
    canvas.translate(roationCenter.dx, roationCenter.dy);
    canvas.rotate((angle / 180) * pi);
    canvas.translate(-roationCenter.dx, -roationCenter.dy);
  }
}

/// Watermark location
enum WMLocation {
  topLeft,
  topCenter,
  topRight,
  centerLeft,
  center,
  centerRight,
  bottomLeft,
  bottomCenter,
  bottomRight,
  tile,
}

==> idkit-0.3.2.tar.gz: lib/src/function/image/func_image_watermark.dart <==
import 'dart:math';
import 'dart:ui';
import 'package:flutter/rendering.dart';
import 'package:idkit/idkit.dart';

/// Image Watermark Extension.
extension WMarkImage on Image {
  /// Image Watermark Extension
  Future<Image> addWaterMarkImage(
    Image wmImg, {
    WMLocation location = WMLocation.tile,
    double opacity = 0.3,
    double sizeScale = 1,
    double vOffset = 20,
    double hOffset = 20,
    double angle = 0,
    bool srcDispose = false,
    bool wmDispose = false,
  }) async {
    final PictureRecorder recorder = PictureRecorder();
    final Canvas canvas = Canvas(recorder, rect);
    canvas.drawImage(this, Offset.zero, Paint());
    _watermakrLayer(
        canvas, wmImg, location, opacity, sizeScale, vOffset, hOffset, angle);
    if (srcDispose) dispose();
    if (wmDispose) wmImg.dispose();
    final Picture picture = recorder.endRecording();
    return picture.toImageSync(width, height);
  }

  /// Image Watermark Extension
  Future<Image> addWaterMarkText(
    String text, {
    WMLocation location = WMLocation.tile,
    TextStyle style = const TextStyle(),
    TextDirection textDirection = TextDirection.ltr,
    int? maxLines,
    double? minWidth,
    double? maxWidth,
    double opacity = 0.3,
    double sizeScale = 1,
    double vOffset = 20,
    double hOffset = 20,
    double angle = 0,
    bool isDispose = false,
  }) async {
    final PictureRecorder recorder = PictureRecorder();
    final Canvas canvas = Canvas(recorder, rect);
    canvas.drawImage(this, Offset.zero, Paint());
    final Image tImage = _textPaint(
      text,
      style,
      textDirection: textDirection,
      minWidth: minWidth ?? 0,
      maxWidth: maxWidth ?? double.infinity,
      maxLines: maxLines ?? 6,
    );
    _watermakrLayer(
        canvas, tImage, location, opacity, sizeScale, vOffset, hOffset, angle);
    if (isDispose) dispose();
    tImage.dispose();
    final Picture picture = recorder.endRecording();
    return picture.toImageSync(width, height);
  }

  /// Image Watermark Extension
  Image addWaterMarkImageSync(
    Image wmImg, {
    WMLocation location = WMLocation.tile,
    double opacity = 0.3,
    double sizeScale = 1,
    double hOffset = 20,
    double vOffset = 20,
    double angle = 0,
    bool srcDispose = false,
    bool wmDispose = false,
  }) {
    final PictureRecorder recorder = PictureRecorder();
    final Canvas canvas = Canvas(recorder, rect);
    canvas.drawImage(this, Offset.zero, Paint());
    _watermakrLayer(
        canvas, wmImg, location, opacity, sizeScale, hOffset, vOffset, angle);
    final Picture picture = recorder.endRecording();
    if (srcDispose) dispose();
    if (wmDispose) wmImg.dispose();
    return picture.toImageSync(width, height);
  }

  /// Image Watermark Extension
  Image addWaterMarkTextSync(
    String text, {
    WMLocation location = WMLocation.tile,
    TextStyle style = const TextStyle(),
    TextDirection textDirection = TextDirection.ltr,
    int? maxLines,
    double? minWidth,
    double? maxWidth,
    double opacity = 0.3,
    double sizeScale = 1,
    double vOffset = 20,
    double hOffset = 20,
    double angle = 0,
    bool isDispose = false,
  }) {
    final PictureRecorder recorder = PictureRecorder();
    final Canvas canvas = Canvas(recorder, rect);
    canvas.drawImage(this, Offset.zero, Paint());
    final Image tImage = _textPaint(
      text,
      style,
      textDirection: textDirection,
      minWidth: minWidth ?? 0,
      maxWidth: maxWidth ?? double.infinity,
      maxLines: maxLines ?? 6,
    );
    _watermakrLayer(
        canvas, tImage, location, opacity, sizeScale, vOffset, hOffset, angle);
    final Picture picture = recorder.endRecording();
    if (isDispose) dispose();
    tImage.dispose();
    return picture.toImageSync(width, height);
  }

  /// Text drawing
  Image _textPaint(
    String text,
    TextStyle style, {
    TextDirection textDirection = TextDirection.ltr,
    double minWidth = 0,
    double maxWidth = double.infinity,
    int maxLines = 6,
  }) {
    final TextPainter painter = TextPainter();
    painter.text = TextSpan(text: text, style: style);
    painter.textDirection = textDirection;
    painter.maxLines = maxLines;
    painter.layout(minWidth: minWidth, maxWidth: maxWidth);
    final Rect tRect = Offset.zero & Size(painter.width, painter.height);
    final PictureRecorder recorder = PictureRecorder();
    final Canvas canvas = Canvas(recorder, tRect);
    canvas.drawRect(tRect, Paint()..color = '#00000000'.aHexColor);
    painter.paint(canvas, Offset.zero);
    final Picture picture = recorder.endRecording();
    return picture.toImageSync(tRect.width.toInt(), tRect.height.toInt());
  }

  /// Watermark layer drawing.
  void _watermakrLayer(
    Canvas canvas,
    Image wmImg,
    WMLocation location,
    double opacity,
    double sizeScale,
    double hOffset,
    double vOffset,
    double angle,
  ) {
    final ColorFilter colorFilter = ColorFilter.matrix(
        [1, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, opacity, 0]);
    final Image hwmImg = wmImg.scaleSync(scale: sizeScale);
    Paint paint = Paint();
    paint.colorFilter = colorFilter;
    switch (location) {
      case WMLocation.topLeft:
        _canvasRotation(canvas, Offset.zero, angle);
        canvas.drawImage(hwmImg, Offset(hOffset, vOffset), paint);
        break;
      case WMLocation.topCenter:
        final double translateDx = width * 0.5 + hOffset;
        _canvasRotation(canvas, Offset(translateDx, vOffset), angle);
        final int wMwidth = hwmImg.width;
        final double dx = (width - wMwidth) * 0.5 + hOffset;
        canvas.drawImage(hwmImg, Offset(dx, vOffset), paint);
        break;
      case WMLocation.topRight:
        final double translateDx = width - hOffset;
        _canvasRotation(canvas, Offset(translateDx, vOffset), angle);
        final int wMwidth = hwmImg.width;
        final double dx = width - (wMwidth + hOffset);
        canvas.drawImage(hwmImg, Offset(dx, vOffset), paint);
        break;
      case WMLocation.centerLeft:
        final double translateDy = height * 0.5 + vOffset;
        _canvasRotation(canvas, Offset(hOffset, translateDy), angle);
        final int wMHeight = hwmImg.height;
        final double dy = (height - wMHeight) * 0.5 + vOffset;
        canvas.drawImage(hwmImg, Offset(hOffset, dy), paint);
        break;
      case WMLocation.center:
        final double translateDx = width * 0.5 + hOffset;
        final double translateDy = height * 0.5 + vOffset;
        _canvasRotation(canvas, Offset(translateDx, translateDy), angle);
        final int wMwidth = hwmImg.width;
        final double dx = (width - wMwidth) * 0.5 + hOffset;
        final int wMHeight = hwmImg.height;
        final double dy = (height - wMHeight) * 0.5 + vOffset;
        canvas.drawImage(hwmImg, Offset(dx, dy), paint);
        break;
      case WMLocation.centerRight:
        final double translateDx = width - hOffset;
        final double translateDy = height * 0.5 + vOffset;
        _canvasRotation(canvas, Offset(translateDx, translateDy), angle);
        final int wMwidth = hwmImg.width;
        final double dx = width - (wMwidth + hOffset);
        final int wMHeight = hwmImg.height;
        final double dy = (height - wMHeight) * 0.5 + vOffset;
        canvas.drawImage(hwmImg, Offset(dx, dy), paint);
        break;
      case WMLocation.bottomLeft:
        final double translateDy = height - vOffset;
        _canvasRotation(canvas, Offset(hOffset, translateDy), angle);
        final int wMHeight = hwmImg.height;
        final double dy = height - (wMHeight + vOffset);
        canvas.drawImage(hwmImg, Offset(hOffset, dy), paint);
        break;
      case WMLocation.bottomCenter:
        final double translateDx = width * 0.5 + hOffset;
        final double translateDy = height - vOffset;
        _canvasRotation(canvas, Offset(translateDx, translateDy), angle);
        final int wMwidth = hwmImg.width;
        final double dx = (width - wMwidth) * 0.5 + hOffset;
        final int wMHeight = hwmImg.height;
        final double dy = height - (wMHeight + vOffset);
        canvas.drawImage(hwmImg, Offset(dx, dy), paint);
        break;
      case WMLocation.bottomRight:
        final double translateDx = width - hOffset;
        final double translateDy = height - vOffset;
        _canvasRotation(canvas, Offset(translateDx, translateDy), angle);
        final int wMwidth = hwmImg.width;
        final double dx = width - (wMwidth + hOffset);
        final int wMHeight = hwmImg.height;
        final double dy = height - (wMHeight + vOffset);
        canvas.drawImage(hwmImg, Offset(dx, dy), paint);
        break;
      default:
        final Matrix4 matrix4 = Matrix4.identity();
        matrix4.rotateZ((angle / 180) * pi);
        matrix4.translate(hOffset, vOffset);
        matrix4.scale(sizeScale, sizeScale);
        paint.shader = ImageShader(
            wmImg, TileMode.repeated, TileMode.repeated, matrix4.storage);
        canvas.drawRect(rect, paint);
    }
    // destroy
    hwmImg.dispose();
  }

  /// Canvas Rotation Handling.
  void _canvasRotation(Canvas canvas, Offset roationCenter, double angle) {
    canvas.translate(roationCenter.dx, roationCenter.dy);
    canvas.rotate((angle / 180) * pi);
    canvas.translate(-roationCenter.dx, -roationCenter.dy);
  }
}

/// Watermark location
enum WMLocation {
  topLeft,
  topCenter,
  topRight,
  centerLeft,
  center,
  centerRight,
  bottomLeft,
  bottomCenter,
  bottomRight,
  tile,
}
